/**
 *  Copyright  2010 Digia Plc
 *  Copyright  2010 Nokia Corporation
 *
 *  All rights reserved.
 *
 *  Nokia and Nokia Connecting People are registered trademarks of
 *  Nokia Corporation.
 *  Java and all Java-based marks are trademarks or registered
 *  trademarks of
 *  Sun Microsystems, Inc. Other product and company names
 *  mentioned herein may be
 *  trademarks or trade names of their respective owners.
 *
 *
 *  Subject to the conditions below, you may, without charge:
 *
 *    Use, copy, modify and/or merge copies of this software and
 *     associated documentation files (the "Software")
 *
 *    Publish, distribute, sub-licence and/or sell new software
 *     derived from or incorporating the Software.
 *
 *
 *  This file, unmodified, shall be included with all copies or
 *  substantial portions
 *  of the Software that are distributed in source code form.
 *
 *  The Software cannot constitute the primary value of any new
 *  software derived
 *  from or incorporating the Software.
 *
 *  Any person dealing with the Software shall not misrepresent
 *  the source of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
 *  KIND, EXPRESS OR IMPLIED,
 *  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY, FITNESS FOR A
 *  PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT
 *  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 *  WHETHER IN AN ACTION
 *  OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 *  CONNECTION WITH THE
 *  SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
import Qt 4.7

Rectangle {
    id: structure

    property CLStyle style: CLStyle {}
    property real fontSize: structure.height * (2/3)
    property color bgColor: style.bgColor
    property color textColor: style.textColor
    property string fontFamily: style.fontFamily
    property string fontWeight: style.fontWeight
    property Gradient nullGradient: Gradient{}
    property bool gradientBgOn: style.gradientBgOn
    property Gradient gradientBg: style.gradientBg
    property int hours: (disablePastTime)? hourPath.currentIndex + currentHours : hourPath.currentIndex
    property int minutes: (disablePastTime && hourPath.currentIndex==0)? minutePath.currentIndex + currentMinutes : minutePath.currentIndex
    property int currentHours: getCurrentHours()
    property int currentMinutes: getCurrentMinutes()
    property bool minutesAffectHours: false
    property bool disablePastTime: false

    /**
     * Gets current hour
     *
     * @return int
     */
    function getCurrentHours() {
        var today = new Date();
        return today.getHours();
    }

    /**
     * Gets current minute
     *
     * @return int
     */
    function getCurrentMinutes() {
        var today = new Date();
        return today.getMinutes();
    }

    /**
     * This function is ran when component is loaded
     *
     * @return Nothing
     */
    function setUpHours() {
        hourPath.currentIndex = 1;
        hourPath.currentIndex = 0;
    }

    /**
     * This function is ran when component is loaded
     *
     * @param
     * @return Nothing
     */
    function setUpMinutes() {
        minutePath.currentIndex = 1;
        minutePath.currentIndex = 0;
    }

    /**
     * Gets selected hour
     *
     * @return Nothing
     */
    function getHours() {
        return hourPath.currentIndex;
    }

    /**
     * Gets selected minute
     *
     * @return Nothing
     */
    function getMinutes() {
        return minutePath.currentIndex;
    }

    /**
     * Decrements selected hour
     *
     * @return Nothing
     */
    function decrementHours() {
        if (hourPath.currentIndex == 0) hourPath.currentIndex = (hourPath.count - 1);
        else hourPath.currentIndex = (hourPath.currentIndex - 1);
        if (hourPath.currentIndex < 0) hourPath.currentIndex = (hourPath.count - 1);
    }

    /**
     * Increments selected hour
     *
     * @return Nothing
     */
    function incrementHours() {
        if (hourPath.currentIndex < (hourPath.count - 1)) hourPath.currentIndex = (hourPath.currentIndex + 1);
        else hourPath.currentIndex = 0;
    }

    /**
     * Decrement selected minute
     *
     * @return Nothing
     */
    function decrementMinutes() {
        if (minutePath.currentIndex == 0) minutePath.currentIndex = (minutePath.count - 1);
        else minutePath.currentIndex = (minutePath.currentIndex - 1);
        if (minutePath.currentIndex < 0) minutePath.currentIndex = (minutePath.count - 1);
        if (minutePath.currentIndex == (minutePath.count - 1)) decrementHours();
    }

    /**
     * Increment selected minute
     *
     * @return Nothing
     */
    function incrementMinutes() {
        if (minutePath.currentIndex < (minutePath.count - 1)) minutePath.currentIndex = (minutePath.currentIndex + 1);
        else minutePath.currentIndex = 0;
        if (minutePath.currentIndex == 0) incrementHours();
    }

    /**
     * NOTE! Functions below require integers. No strings or leading-zeros!
     */

    /**
     * Set selected hours
     *
     * @param newHours New value for selected hour
     * @return Nothing
     */
    function setHours(newHours) {
        if (newHours < 0) newHours = 0;
        if (newHours > 23) newHours = 23;
        if (structure.disablePastTime && newHours <= currentHours) {
            hourPath.currentIndex = 0;
        } else if (structure.disablePastTime && newHours > currentHours) {
            hourPath.currentIndex = newHours - currentHours;
        } else {
            hourPath.currentIndex = newHours;
        }
    }

    /**
     * Set selected minute
     *
     * @param newMinutes New value for selected minute
     * @return Nothing
     */
    function setMinutes(newMinutes) {
        if (newMinutes < 0) newMinutes = 0;
        if (newMinutes > 59) newMinutes = 59;
        if (structure.disablePastTime && hourPath.currentIndex == 0 && newMinutes <= currentMinutes) {
            minutePath.currentIndex = 0;
        } else if (structure.disablePastTime && hourPath.currentIndex == 0 && newMinutes > currentMinutes) {
            minutePath.currentIndex = newMinutes - currentMinutes;
        } else {
            minutePath.currentIndex = newMinutes;
        }
    }

    /**
     * Set selected time
     *
     * @param newHours New value for selected hour
     * @param newMinutes New value for selected minute
     * @return Nothing
     */
    function setTime (newHours, newMinutes) {
        setHours(newHours);
        setMinutes(newMinutes);
    }


    onWidthChanged: {
        var totalWidth = 2*measureText.width + doubleDot.width;
        if (structure.width < totalWidth) structure.width = totalWidth;
    }

    Component.onCompleted: {
        var totalWidth = 2*measureText.width + doubleDot.width;
        if (structure.width < totalWidth) structure.width = totalWidth;
        incrementMinutes();
        decrementMinutes();
        incrementHours();
        decrementHours();
    }

    width: 500
    height: 300
    clip: true
    color: structure.bgColor
    gradient: (gradientBgOn)? gradientBg : nullGradient

    Component {
        id: hourDelegate

        Item {
            property int thisValue: (disablePastTime)? index + structure.currentHours : index
            width: structure.width / 2
            height: structure.fontSize
            opacity: PathView.opacity

            Text {
                anchors.centerIn: parent
                text: (parent.thisValue < 10)? "0"+parent.thisValue : parent.thisValue
                font.pixelSize: structure.fontSize
                elide: Text.ElideMiddle
                color: structure.textColor
                font.family: structure.fontFamily
                font.weight: structure.fontWeight
            }

            MouseArea {
                onClicked: if (!hourPath.moving) incrementHours();
                anchors.fill: parent
            }
        }
    }

    Component {
        id: minuteDelegate

        Item {
            property int thisValue: (disablePastTime && hourPath.currentIndex==0)? index + structure.currentMinutes : index
            width: structure.width / 2
            height: structure.fontSize
            opacity: PathView.opacity

            Text {
                anchors.centerIn: parent
                text: (parent.thisValue < 10)? "0"+parent.thisValue : parent.thisValue
                font.pixelSize: structure.fontSize
                elide: Text.ElideMiddle
                color: structure.textColor
                font.family: structure.fontFamily
                font.weight: structure.fontWeight
            }

            MouseArea {
                onClicked: if (!minutePath.moving) incrementMinutes();
                anchors.fill: parent
            }
        }
    }
    
    onDisablePastTimeChanged: {
        incrementMinutes();
        decrementMinutes();
        incrementHours();
        decrementHours();
    }

    Text {
        id: measureText

        text: "00"
        font.pixelSize: structure.fontSize
        font.family: structure.fontFamily
        font.weight: structure.fontWeight
        visible: false
    }

    Text {
        id: doubleDot

        anchors.centerIn: parent
        text: ":"
        font.pixelSize: structure.fontSize
        color: structure.textColor
        font.family: structure.fontFamily
        font.weight: structure.fontWeight
    }

    PathView {
        id: hourPath

        Component.onCompleted: setUpHours()
        onCountChanged: {
            if (structure.disablePastTime && hourPath.currentIndex >= hourPath.count) hourPath.currentIndex = hourPath.count-1;
            if (structure.disablePastTime && minutePath.currentIndex >= minutePath.count) minutePath.currentIndex = minutePath.count-1;
            incrementHours();
            decrementHours();
        }
        x: 0
        y: 0
        model: (structure.disablePastTime)? 24-structure.currentHours : 24
        width: structure.width/2
        height: structure.height
        delegate: hourDelegate
        pathItemCount: 5
        highlightRangeMode: PathView.StrictlyEnforceRange
        preferredHighlightBegin: 0.5
        preferredHighlightEnd: 0.5
        interactive: true
        path: Path {
            startX: Math.round(structure.width/4); startY: Math.round(structure.height+1.5*structure.fontSize)
            PathAttribute { name: "type"; value: 0 }
            PathAttribute { name: "opacity"; value: 0 }
            PathLine { x: Math.round(structure.width/4); y: Math.round((3/4)*structure.height) }
            PathAttribute { name: "opacity"; value: 0.4 }
            PathLine { x: Math.round(structure.width/4); y: Math.round((2/4)*structure.height) }
            PathAttribute { name: "opacity"; value: 1 }
            PathLine { x: Math.round(structure.width/4); y: Math.round((1/4)*structure.height) }
            PathAttribute { name: "opacity"; value: 0.4 }
            PathLine { x: Math.round(structure.width/4); y: Math.round(-1.5*structure.fontSize) }
            PathAttribute { name: "opacity"; value: 0 }
            PathAttribute { name: "type"; value: 0 }
        }
    }

    PathView {
        id: minutePath

        Component.onCompleted: setUpMinutes()
        onCountChanged: {
            incrementMinutes();
            decrementMinutes();
        }

        x: structure.width/2
        y: 0
        model: (structure.disablePastTime && hourPath.currentIndex==0)? 60-structure.currentMinutes : 60
        width: structure.width/2
        height: structure.height
        delegate: minuteDelegate
        pathItemCount: 5
        highlightRangeMode: PathView.StrictlyEnforceRange
        preferredHighlightBegin: 0.5
        preferredHighlightEnd: 0.5
        interactive: true
        path: Path {
            startX: Math.round(structure.width/4); startY: Math.round(structure.height+1.5*structure.fontSize)
            PathAttribute { name: "type"; value: 1 }
            PathAttribute { name: "opacity"; value: 0 }
            PathLine { x: Math.round(structure.width/4); y: Math.round((3/4)*structure.height) }
            PathAttribute { name: "opacity"; value: 0.4 }
            PathLine { x: Math.round(structure.width/4); y: Math.round((2/4)*structure.height) }
            PathAttribute { name: "opacity"; value: 1 }
            PathLine { x: Math.round(structure.width/4); y: Math.round((1/4)*structure.height) }
            PathAttribute { name: "opacity"; value: 0.4 }
            PathLine { x: Math.round(structure.width/4); y: Math.round(-1.5*structure.fontSize) }
            PathAttribute { name: "opacity"; value: 0 }
            PathAttribute { name: "type"; value: 1 }
        }
    }
}
